//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Fri Jan  8 08:33:57 PST 1999
// Last Modified: Fri Jan  8 08:34:01 PST 1999
// Filename:      ...sig/code/control/MidiInPort/linux/MidiInPort_linux.h
// Syntax:        C++ 
// $Smake:        g++ -g -c %b.cpp -I../../../../include && rm -f %b.o
//

#ifndef _MIDIINPUT_LINUX_H_INCLUDED
#define _MIDIINPUT_LINUX_H_INCLUDED

#ifdef LINUX

#include "MidiMessage.h"
#include "CircularBuffer.h"
#include "Sequencer_linux.h"
#include "SigTimer.h"
#include <pthread.h>

typedef unsigned char uchar;


class MidiInPort_linux : public Sequencer_linux {
   public:
                      MidiInPort_linux           (void);
                      MidiInPort_linux           (int aPort, int autoOpen = 1);
                     ~MidiInPort_linux           ();

      void            close                      (void);
      void            closeAll                   (void);
      MidiMessage     extract                    (void);
      int             getBufferSize              (void);
      int             getChannelOffset           (void) const;
      int             getCount                   (void);
      const char*     getName                    (void);
      static const char* getName                 (int i);
      static int      getNumPorts                (void);
      int             getPort                    (void);
      int             getPortStatus              (void);
      int             getTrace                   (void);
      void            insert                     (const MidiMessage& aMessage);
      MidiMessage&    message                    (int index);
      int             open                       (void);
      void            pause                      (void);
      void            setBufferSize              (int aSize);
      void            setChannelOffset           (int anOffset);
      void            setPort                    (int aPort);
      int             setTrace                   (int aState);
      void            toggleTrace                (void);
      void            unpause                    (void);

   protected:
      int    port;     // the port to which this object belongs
 
      static int        objectCount;     // num of similar objects in existence
      static int*       portObjectCount; // objects connected to particular port
      static int*       trace;           // for verifying input
      static ostream*   tracedisplay;    // stream for displaying trace
      static int        numDevices;      // number of input ports
      static CircularBuffer<MidiMessage>** midiBuffer; // MIDI storage frm ports
      static int        channelOffset;   // channel offset, either 0 or 1
                                         // not being used right now.
      static int*       pauseQ;          // for adding items to Buffer or not
      static SigTimer   midiTimer;       // for timing MIDI input
      static pthread_t  midiInThread;    // for MIDI input thread function

   private:
      void            deinitialize               (void); 
      void            initialize                 (void); 

 
   friend void *interpretMidiInputStreamPrivate(void * x);
    
};


#endif  // LINUX

#endif  // _MIDIINPUT_LINUX_H_INCLUDED



