//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Sun Jun  7 16:24:06 PDT 1998
// Last Modified: Sun Jun  7 20:49:27 PDT 1998
// Filename:      ...humdrum++/src/ChordQuality.cpp
// Syntax:        C++ 
// $Smake:        cc -Wall -I../include -g -c %f && rm %b.o
//


#include "ChordQuality.h"
#include "Convert.h"
#include "HumdrumEnumerations.h"
#include "function_information.h"
#include <strstream.h>


char* ChordQuality::displayString = NULL;



//////////////////////////////
//
// ChordQuality
//

ChordQuality::ChordQuality(void) {
   function_information("ChordQuality::ChordQuality(void)")

   chordType = E_unknown;
   chordInversion = E_unknown;
   chordRoot = E_unknown;
   if (getDisplay() == NULL) {
      setDisplay("t:i:r");
   }
}


ChordQuality::ChordQuality(const ChordQuality& aChordQuality) {
   function_information("ChordQuality::ChordQuality(const ChordQuality&)")
   chordType = aChordQuality.getType();;
   chordInversion = aChordQuality.getInversion();;
   chordRoot = aChordQuality.getRoot();
}


ChordQuality::ChordQuality(int aType, int anInversion, int aRoot) {
   function_information("ChordQuality::ChordQuality(int, int, int)")

   chordType = aType;
   chordInversion = anInversion;
   chordRoot = aRoot;
}



//////////////////////////////
//
// ChordQuality::~ChordQuality
//

ChordQuality::~ChordQuality() {
   function_information("ChordQuality::~ChordQuality")

   chordType = E_unknown;
   chordInversion = E_unknown;
   chordRoot = E_unknown;
}



//////////////////////////////
//
// ChordQuality::getBassNote
//

int ChordQuality::getBassNote(void) const {
   function_information("ChordQuality::getBassNote")

   return Convert::chordQualityToBaseNote(*this);
}



//////////////////////////////
//
// ChordQuality::getDisplay
//

const char* ChordQuality::getDisplay(void) {
   function_information("ChordQuality::getDisplay")

   return displayString;
}



//////////////////////////////
//
// ChordQuality::getInversion
//

int ChordQuality::getInversion(void) const {
   function_information("ChordQuality::getInversion")

   return chordInversion;
}



//////////////////////////////
//
// ChordQuality::getInversionName
//

const char* ChordQuality::getInversionName(void) const {
   function_information("ChordQuality::getInversionName")

   return Convert::chordInversion.getName(getInversion());
}



//////////////////////////////
//
// ChordQuality::getNotesInChord
//

Collection<int> ChordQuality::getNotesInChord(void) const {
   function_information("ChordQuality::getNotesInChord")

   return Convert::chordQualityToNoteSet(*this);
}



//////////////////////////////
//
// ChordQuality::getRoot
//

int ChordQuality::getRoot(void) const {
   function_information("ChordQuality::getRoot")

   return chordRoot;
}



//////////////////////////////
//
// ChordQuality::getRootName
//

const char* ChordQuality::getRootName(void) const {
   function_information("ChordQuality::getRootName")

   return Convert::kernPitchClass.getName(getRoot());
}



//////////////////////////////
//
// ChordQuality::getType
//

int ChordQuality::getType(void) const {
   function_information("ChordQuality::getType")

   return chordType;
}



//////////////////////////////
//
// ChordQuality::getTypeName
//

const char* ChordQuality::getTypeName(void) const {
   function_information("ChordQuality::getName")

   return Convert::chordType.getName(getType());
}



//////////////////////////////
//
// ChordQuality::makeString
//

void ChordQuality::makeString(char* space) {
   function_information("ChordQuality::makeString")

   strstream temp;
   temp << *this << ends;
   strcpy(space, temp.str());
}



//////////////////////////////
//
// ChordQuality::print
//

void ChordQuality::print(const char* aDisplayString, ostream& out) const {
   function_information("ChordQuality::print(const char*, ostream&)")

   int i = 0;
   while (aDisplayString[i] != '\0') {
      switch (aDisplayString[i]) {
         case 't':
            out << getTypeName();
            break;
         case 'i':
            if (getType() == E_chord_note) {
               ;  // do nothing
            } else {
               out << getInversionName();
            }
            break;
         case 'r':
            out << getRootName();
            break;
         default:
            out << aDisplayString[i];
      }
      i++;
   }
}


void ChordQuality::print(ostream& out) const {
   function_information("ChordQuality::print(ostream&)")

   print(getDisplay(), out);
}



//////////////////////////////
//
// ChordQuality::setDisplay
//

void ChordQuality::setDisplay(const char* aDisplayFormat) {
   function_information("ChordQuality::setDisplay")

   if (displayString != NULL) {
      delete [] displayString;
   }
   displayString = new char[strlen(aDisplayFormat) + 1];
   strcpy(displayString, aDisplayFormat);
}



//////////////////////////////
//
// ChordQuality::setInversion
//

void ChordQuality::setInversion(int anInversion) {
   function_information("ChordQuality::setInversion(int)")

   chordInversion = anInversion;
}


void ChordQuality::setInversion(const char* anInversionName) {
   function_information("ChordQuality::setInversion(const char*)")

   chordInversion = Convert::chordInversion.getValue(anInversionName);
}



//////////////////////////////
//
// ChordQuality::setRoot
//

void ChordQuality::setRoot(int aRoot) {
   function_information("ChordQuality::setRoot(int)")

   chordRoot = aRoot;
}


void ChordQuality::setRoot(const char* aRoot) {
   function_information("ChordQuality::setRoot(const char*)")

   chordRoot = Convert::kernPitchClass.getValue(aRoot);
}



//////////////////////////////
//
// ChordQuality::setQuality
//

void ChordQuality::setQuality(const char* aQuality) {
   function_information("ChordQuality::setQuality")

   chordType = Convert::chordQualityToType(aQuality);
   chordInversion = Convert::chordQualityToInversion(aQuality);
   chordRoot = Convert::chordQualityToRoot(aQuality);
}



//////////////////////////////
//
// ChordQuality::setType
//

void ChordQuality::setType(int aType) {
   function_information("ChordQuality::setType(int)")

   chordType = aType;
}


void ChordQuality::setType(const char* aTypeName) {
   function_information("ChordQuality::setType(const char*)")

   chordType = Convert::chordType.getValue(aTypeName);
}



///////////////////////////////////////////////////////////////////////////
//
// external functions
//


//////////////////////////////
//
// operator << 
//

ostream& operator<<(ostream& out, const ChordQuality& aChordQuality) {
   function_information("operator<<(ostream&, const ChordQuality&)")

   aChordQuality.print(out);
   return out;
}




