//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Mon May 18 13:43:47 PDT 1998
// Last Modified: Wed Jun 10 22:49:12 PDT 1998
// Filename:      ...humdrum++/src/HumdrumFile.cpp
// Syntax:        C++ 
// $Smake:        cc -Wall -I../include -g -c %f && rm %b.o
//


#include "HumdrumFile.h"
#include "function_information.h"
#include "Convert.h"
#include <fstream.h>
#include <iostream.h>
#include <stdlib.h>



//////////////////////////////
//
// HumdrumFile::HumdrumFile
//

HumdrumFile::HumdrumFile(void) {
   function_information("HumdrumFile::HumdrumFile(void)")

   records.setSize(1000);          // initial storage size 1000 lines
   records.setSize(0);
   records.allowGrowth();          
   records.setAllocSize(1000);     // grow in increments of 1000 lines
}


HumdrumFile::HumdrumFile(const HumdrumFile& aHumdrumFile) {
   function_information("HumdrumFile::HumdrumFile(const HumdrumFile&)")

   records.setSize(1000);          // initial storage size 1000 lines
   records.setSize(0);
   records.allowGrowth();          
   records.setAllocSize(1000);     // grow in increments of 1000 lines

   *this = aHumdrumFile;
}


HumdrumFile::HumdrumFile(const char* filename) {
   function_information("HumdrumFile::HumdrumFile(const char*)")

   records.setSize(1000);          // initial storage size 1000 lines
   records.setSize(0);
   records.allowGrowth();          
   records.setAllocSize(1000);     // grow in increments of 1000 lines

   ifstream infile(filename, ios::in | ios::nocreate);
   if (!infile) {
      cerr << "Error: cannot open file: " << filename << endl;
      exit(1);
   }

   char templine[4096];
   while (!infile.eof()) {
      infile.getline(templine, 4096);
      appendLine(templine);
   }
   assignExInt();
}



//////////////////////////////
//
// HumdrumFile::~HumdrumFile
//

HumdrumFile::~HumdrumFile() {
   function_information("HumdrumFile::~HumdrumFile")

   clear();
}



//////////////////////////////
//
// HumdrumFile::appendLine  -- adds a line to a humdrum file
//

void HumdrumFile::appendLine(const char* aLine) {
   function_information("HumdrumFile::appendLine(const char*)")

   HumdrumRecord* aRecord;
   aRecord = new HumdrumRecord;
   aRecord->setLine(aLine);
   records[records.getSize()] = aRecord;
}


void HumdrumFile::appendLine(HumdrumRecord& aRecord) {
   function_information("HumdrumFile::appendLine(HumdrumRecord&)")

   HumdrumRecord *tempRecord;
   tempRecord = new HumdrumRecord;
   *tempRecord = aRecord;
   records[records.getSize()] = tempRecord;
}
   

void HumdrumFile::appendLine(HumdrumRecord* aRecord) {
   function_information("HumdrumFile::appendLine(HumdrumRecord*)")

   appendLine(*aRecord);
}



//////////////////////////////
//
// Humdrum::assignExInt -- traces through spines and
//	marks the exclusive interpretations for each data field.
//	run this function when you want the exint markers
//	for the line fields to be accurate.
//
// Note: currently does not allow splitting or joining of
// 	spines.  Will be done later
//

void HumdrumFile::assignExInt(void) {
   function_information("HumdrumFile::assignExInt")

   Collection<int> interps;
   int i, k;
   int location = -1;
   int endlocation;
   int lastSize = 0;
   int setupQ = 0;
   for (i=0; i<records.getSize(); i++) {
      if (!setupQ && records[i]->getType() == E_humrec_data_interpretation) {
         interps.setSize(records[i]->getFieldCount());
         for (k=0; k<records[i]->getFieldCount(); k++) {
            interps[k] = Convert::exint.getValue((*records[i])[k]);
         }
         setupQ = 1;
         lastSize = interps.getSize();
      }

      // handle assignment of excluisve interpretations
      if (setupQ && records[i]->getType() >= E_humrec_data) {
         // handle splitting or joining of spines
         if ((*records[i])[0][0] == '*') {
            for (int p=0; p<records[i]->getFieldCount(); p++) {
               // spine expansion: *^
               if (strcmp((*records[i])[p], "*^") == 0) {
                  location = -1;
                  for (int m=0; m<records[i]->getFieldCount(); m++) {
                     if (strcmp((*records[i])[m], "*^") == 0) {
                        location = m;
                        interps.setSize(interps.getSize() + 1);
                        for (int n=interps.getSize() - 1; n>location; n--) {
                           interps[n] = interps[n-1];
                        }
                        break;
                     }
                  }
                  break;
               // spine contractions: *v
               } else if (strcmp((*records[i])[p], "*v") == 0) {
                  location = -1;
                  for (int m=0; m<records[i]->getFieldCount(); m++) {
                     if (strcmp((*records[i])[m], "*v") == 0) {
                        location = m;
                        endlocation = m;
                        while (endlocation < records[i]->getFieldCount() &&
                              strcmp((*records[i])[endlocation],"*v") == 0) {
                           endlocation++;
                        }
                        endlocation--;
                        for (int n=location; 
                              n<interps.getSize()-1-endlocation+location;
                              n++) {
                           interps[n] = interps[n+endlocation-location];
                        }
                        interps.setSize(interps.getSize() - 
                              1 - location + endlocation);
                        break;
                     }
                  }
                  break;
               }

            }
         }
 
         for (k=0; k<records[i]->getFieldCount(); k++) {
            records[i]->setExInt(k, interps[k]);
         }
      }
   }
}      
      


//////////////////////////////
//
// HumdrumFile::clear -- removes all lines from the humdrum file
//

void HumdrumFile::clear(void) {
   function_information("HumdrumFile::clear")

   for (int i=0; i<getNumLines(); i++) {
      delete records[i];
      records[i] = NULL;
   }
   records.setSize(0);
}



//////////////////////////////
//
// HumdrumFile::extract
//

HumdrumFile HumdrumFile::extract(int aField) {
   function_information("HumdrumFile::extract")

   int fieldNumber = aField;
   HumdrumRecord aRecord;
   HumdrumFile output;
   for (int i=0; i<getNumLines(); i++) {
      switch(records[i]->getType()) {
         case E_humrec_data:
         case E_humrec_data_comment:
         case E_humrec_data_kern_measure:
         case E_humrec_data_tandem:
         case E_humrec_data_interpretation:
            if (fieldNumber < 0) {
               fieldNumber = records[i]->getFieldCount() - 1;
            }
            aRecord.setLine((*records[i])[fieldNumber]);
            aRecord.setExInt(0, records[i]->getExInt(fieldNumber));
            output.appendLine(aRecord);
            break;
         case E_humrec_empty:
            output.appendLine(records[i]);
            break;
         case E_humrec_none:
         case E_humrec_comment:
         case E_humrec_global_comment:
         default:
            output.appendLine(records[i]);
      }
   }

   return output;
}



//////////////////////////////
//
// HumdrumFile::getLine -- returns the character string of the
//	specified line
//

const char* HumdrumFile::getLine(int index) {
   function_information("HumdrumFile::getLine")

   if (index >= getNumLines()) {
      cerr << "Error: maximum line index is: " << getNumLines() 
           << ", but you asked for line index: " << index << endl;
      exit(1);
   }
  
   return records[index]->getLine();
}



//////////////////////////////
//
// HumdrumFile::getNumLines  -- returns the number of lines
//	in the HumdrumFile
//

int HumdrumFile::getNumLines(void) {
   function_information("HumdrumFile::getNumLines")

   return records.getSize();
}



//////////////////////////////
//
// HumdrumFile::getRecord  -- returns the record of the specified line
//

HumdrumRecord& HumdrumFile::getRecord(int index) {
   function_information("HumdrumFile::getRecord")

   if (index >= getNumLines()) {
      cerr << "Error: maximum line index is: " << getNumLines() 
           << ", but you asked for line index: " << index << endl;
      exit(1);
   }

   return *records[index];
}



//////////////////////////////
//
// HumdrumFile::operator=
//

HumdrumFile& HumdrumFile::operator=(const HumdrumFile& aFile) {
   function_information("HumdrumFile::operator=")

   // don't copy onto self
   if (&aFile == this) {
      return *this;
   }

   // delete any current contents
   int i;
   for (i=0; i<records.getSize(); i++) {
      delete records[i];
      records[i] = NULL;
   }

   records.setSize(aFile.records.getSize());
   for (i=0; i<aFile.records.getSize(); i++) {
      records[i] = new HumdrumRecord;
      *(records[i]) = *(aFile.records[i]);
   }

   return *this;
}



//////////////////////////////
//
// HumdrumFile::read -- read in a humdrum file.
//

void HumdrumFile::read(const char* filename) {
   function_information("HumdrumFile::read(const char*)")

   int i;
   for (i=0; i<records.getSize(); i++) {
      delete records[i];
      records[i] = NULL;
   }
   records.setSize(0);

   ifstream infile(filename, ios::in | ios::nocreate);
   if (!infile) {
      cerr << "Error: could not open file: " << filename << endl;
      exit(1);
   }

   char templine[4096];
   while (!infile.eof()) {
      infile.getline(templine, 4096, '\n');
      appendLine(templine);
   }
   assignExInt();
}


void HumdrumFile::read(istream& inStream) {
   function_information("HumdrumFile::read(istream&)")
   char* templine;
   templine = new char[4096];
   while (!inStream.eof()) {
      inStream.getline(templine, 4096);
      appendLine(templine);
   }
   assignExInt();
   delete [] templine;
}



//////////////////////////////
//
// HumdrumFile::removeNullRecords
//

HumdrumFile HumdrumFile::removeNullRecords(void) {
   function_information("HumdrumFile::removeNullRecords")

   HumdrumFile output;
   int j;
   int nulltest = 1;
   for (int i=0; i<getNumLines(); i++) {
      nulltest = 1;
      switch(records[i]->getType()) {
         case E_humrec_data:
            for (j=0; j<records[i]->getFieldCount(); j++) {
               if (strcmp((*records[i])[j], ".") != 0) {
                  nulltest = 0;
                  break;
               }
            }
            if (nulltest) {
               // do nothing
            } else {
               output.appendLine(records[i]);
            }
            break;
         default:
            output.appendLine(records[i]);
      }
   }

   return output;
}





















//////////////////////////////
//
// HumdrumFile::operator[]
//

HumdrumRecord& HumdrumFile::operator[](int index) {
   function_information("HumdrumFile::operator[]")

   if (index >= getNumLines()) {
      cerr << "Error: maximum line index is: " << getNumLines() 
           << ", but you asked for line index: " << index << endl;
      exit(1);
   }

   return *records[index];
}



//////////////////////////////
//
// HumdrumFile::write -- write out a humdrum file.
//

void HumdrumFile::write(const char* filename) {
   function_information("HumdrumFile::write(const char*)")

   ofstream outfile(filename, ios::out | ios::noreplace);
   if (!outfile) {
      cerr << "Error: could not open file for writing: " << filename << endl;
      cerr << "Perhaps it already exists?" << endl;
      exit(1);
   }

   for (int i=0; i<records.getSize(); i++) {
      outfile << records[i]->getLine() << endl;
   }
}


void HumdrumFile::write(ostream& outStream) {
   function_information("HumdrumFile::write(ostream&)")

   for (int i=0; i<records.getSize(); i++) {
      outStream << records[i]->getLine() << endl;
   }
}



///////////////////////////////////////////////////////////////////////////



//////////////////////////////
//
// operator<<
//

ostream& operator<<(ostream& out, HumdrumFile& aHumdrumFile) {
   function_information("operator<<(ostream&, HumdrumFile&)")

   for (int i=0; i<aHumdrumFile.getNumLines(); i++) {
      out << aHumdrumFile[i].getLine() << '\n';
   }
   return out;
}



